<?php

/**
 *
 * VM Payment - PayTR Virtual Pos iFrame API
 *
 * @author PayTR Ödeme Hizmetleri A.Ş.
 * @package VirtueMart
 * @subpackage payment
 * Copyright (C) 2021 PayTR Ödeme Hizmetleri A.Ş. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
 *
 * http://virtuemart.net
 */

defined('_JEXEC') or die('Restricted access');

/**
 * Class PaytrHelper
 */
class PaytrHelper
{

    protected $method;
    protected $order;

    /**
     * PaytrHelper constructor.
     * @param $method
     * @param null $order
     */
    public function __construct($method, $order = null)
    {
      $this->method = $method;
      $this->order  = $order;
    }

    /**
     * @param $installments
     * @return array|string[]
     */
    private function installmentCalculate($installments)
    {
        if(in_array('1', $installments))
        {
          return [
            'no_installment'    => "1",
            'max_installment'   => "0",
          ];
        }
        elseif (($key = array_search('0', $installments)) !== false && count($installments) > 1)
        {
          unset($installments[$key]);
          return [
            'no_installment'    => "0",
            'max_installment'   => min($installments),
          ];
        }
        return [
          'no_installment'    => "0",
          'max_installment'   => count($installments) ? min($installments) : '0',
        ];
    }

    /**
     * @return mixed
     */
    public function getMerchantID()
    {
        return $this->method->paytr_merchant_id;
    }

    /**
     * @return mixed
     */
    public function getMerchantKey()
    {
        return $this->method->paytr_merchant_key;
    }

    /**
     * @return mixed
     */
    public function getMerchantSalt()
    {
        return $this->method->paytr_merchant_salt;
    }

    /**
     * @return int|string|null
     */
    public function getCurrency()
    {
        $currency = CurrencyDisplay::getInstance($this->order['details']['BT']->order_currency_id)->_vendorCurrency_code_3;
        if($currency === 'TRY')
        {
          return 'TL';
        }
        return $currency;
    }

    /**
     * @return float|int
     */
    public function getTotalAmount()
    {
        return str_replace(['.', ','], '', number_format($this->order['details']['BT']->order_total, '2'));
    }

    /**
     * @return mixed
     */
    public function getCustomerEmail()
    {
        return $this->order['details']['BT']->email;
    }

    /**
     * @return string
     */
    public function getNameSurname()
    {
        $details = $this->order['details']['BT'];
        return $details->first_name.' '.$details->middle_name.' '.$details->last_name;
    }

    /**
     * @return string
     */
    public function getAddress()
    {
        $details = $this->order['details']['BT'];
        return $details->address_1.' '.$details->address_2.' / '.$details->city.' / '.ShopFunctions::getCountryByID($details->virtuemart_country_id);
    }

    /**
     * @return string
     */
    public function getUserBasket()
    {
        $items = [];
        foreach ($this->order['items'] as $key => $item)
        {
          $items[] = $item->order_item_name;
          $items[] = number_format($item->product_item_price, 2);
          $items[] = round($item->product_quantity);
        }
        return base64_encode(json_encode([$items]));
    }

    /**
     * @return mixed
     */
    public function getPhone()
    {
        $details = $this->order['details']['BT'];
        return $details->phone_2 ? $details->phone_2 : $details->phone_1;
    }

    public function getMerchantOkUrl()
    {
        return JURI::root() . "index.php?option=com_virtuemart&view=pluginresponse&task=pluginresponsereceived&pm=" . $this->order['details']['BT']->virtuemart_paymentmethod_id.'&on='.$this->order['details']['BT']->order_number;
    }

    public function getMerchantFailUrl()
    {
        return 'http://joomla.test/index.php/cart/orderdone';
    }

    /**
     * @return mixed
     */
    public function getUserIpAddress()
    {
        if( isset( $_SERVER["HTTP_CLIENT_IP"] ) ) {
          $ip = $_SERVER["HTTP_CLIENT_IP"];
        } elseif( isset( $_SERVER["HTTP_X_FORWARDED_FOR"] ) ) {
          $ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
        } else {
          $ip = $_SERVER["REMOTE_ADDR"];
        }
        return $ip;
    }

    /**
     * @return array|string[]
     */
    public function getInstallments()
    {
        return $this->getCategoryBasedInstallments();
    }

    /**
     * @return string
     */
    public function getMerchantOid()
    {
        return 'SP'.$this->order['details']['BT']->order_number.'VM'.time();
    }

    /**
     * @return mixed
     */
    public function getTestMode()
    {
        return $this->method->paytr_test_mode;
    }

    /**
     * @return string
     */
    public function getLang()
    {
        $lang = (JFactory::getLanguage())->getTag();
        return !explode('-', $lang)[0] === 'tr' ? 'tr' : 'en';
    }

    /**
     * @return array|string[]
     */
    private function getCategoryBasedInstallments()
    {
        $current_installments = [];
        $category_installments = [];

        foreach ($this->method->category_based_installment as $key => $installment)
        {
            $current_installments[$installment->category] = $installment->installment_count;
        }
        foreach ($this->order['items'] as $key => $item)
        {
            if(array_key_exists($item->virtuemart_category_id, $current_installments))
            {
              $category_installments[$item->virtuemart_category_id] = $current_installments[$item->virtuemart_category_id];
            }
        }
        return $this->installmentCalculate($category_installments);
    }

    /**
     * @return string
     */
    public function makeHashStr()
    {
        return
          $this->getMerchantId()
          . $this->getUserIpAddress()
          . $this->getMerchantOid()
          . $this->getCustomerEmail()
          . $this->getTotalAmount()
          . $this->getUserBasket()
          . $this->getInstallments()['no_installment']
          . $this->getInstallments()['max_installment']
          . $this->getCurrency()
          . $this->getTestMode();
    }

    /**
     * @return string
     */
    public function getToken()
    {
        return base64_encode(hash_hmac('sha256', $this->makeHashStr().$this->getMerchantSalt(), $this->getMerchantKey(), true));
    }

    /**
     * @return array
     */
    public function makePostVariables()
    {
        return array(
          'merchant_id'       =>  $this->getMerchantId(),
          'user_ip'           =>  $this->getUserIpAddress(),
          'merchant_oid'      =>  $this->getMerchantOid(),
          'email'             =>  $this->getCustomerEmail(),
          'payment_amount'    =>  $this->getTotalAmount(),
          'paytr_token'       =>  $this->getToken(),
          'user_basket'       =>  $this->getUserBasket(),
          'debug_on'          =>  1,
          'no_installment'    =>  $this->getInstallments()['no_installment'],
          'max_installment'   =>  $this->getInstallments()['max_installment'],
          'user_name'         =>  $this->getNameSurname(),
          'user_address'      =>  $this->getAddress(),
          'user_phone'        =>  $this->getPhone(),
          'merchant_ok_url'   =>  $this->getMerchantOkUrl(),
          'merchant_fail_url' =>  $this->getMerchantFailUrl(),
          'timeout_limit'     =>  30,
          'currency'          =>  $this->getCurrency(),
          'test_mode'         =>  $this->getTestMode(),
          'lang'              =>  $this->getLang()
        );
    }
}
