<?php

/**
 *
 * VM Payment - PayTR Virtual Pos iFrame API
 *
 * @author PayTR Ödeme Hizmetleri A.Ş.
 * @package VirtueMart
 * @subpackage payment
 * Copyright (C) 2021 PayTR Ödeme Hizmetleri A.Ş. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
 *
 * http://virtuemart.net
 */

defined ('_JEXEC') or die('Restricted access');
if (!class_exists ('vmPSPlugin')) {
    require(VMPATH_PLUGINLIBS .'/vmpsplugin.php');
}

class plgVmpaymentPaytr extends vmPSPlugin {

    protected $paymentMethod;

    protected $pluginConfig;

    function __construct (&$subject, $config) {
        parent::__construct ($subject, $config);
        if (!class_exists('PaytrHelper'))
        {
            $path = VMPATH_ROOT .'/plugins/vmpayment/paytr/helpers/PaytrHelper.php';
            if(file_exists($path)) require $path;
        }
        if (!class_exists('PaytrRequestHelper'))
        {
            $path = VMPATH_ROOT .'/plugins/vmpayment/paytr/helpers/PaytrRequestHelper.php';
            if(file_exists($path)) require $path;
        }
        $jlang              = JFactory::getLanguage();
        $jlang->load ('plg_vmpayment_paytr', JPATH_ADMINISTRATOR, NULL, TRUE);
        $this->tableFields  = array_keys($this->getTableSQLFields());
        $this->_loggable    = TRUE;
        $this->_debug       = TRUE;
        $this->_tablepkey   = 'id';
        $this->_tableId     = 'id';
        $varsToPush         = $this->getVarsToPush();
        $this->addVarsToPushCore($varsToPush, 1);
        $this->setConfigParameterable ($this->_configTableFieldName, $varsToPush);
    }

    public function getVarsToPush()
    {
        $varsToPush = parent::getVarsToPush();
        $varsToPush['hide_login'] = array(0, 'int');
        return $varsToPush;
    }

    public function getVmPluginCreateTableSQL () {

        return $this->createTableSQL ('PayTR Payment Table');
    }

    function _processStatus (&$mb_data, $method) {
        switch ($mb_data['status']) {
            case 2 :
                $mb_data['payment_status'] = 'Completed';
                break;
            case 0 :
                $mb_data['payment_status'] = 'Pending';
                break;
            case -1 :
                $mb_data['payment_status'] = 'Cancelled';
                break;
            case -2 :
                $mb_data['payment_status'] = 'Failed';
                break;
            case -3 :
                $mb_data['payment_status'] = 'Chargeback';
                break;
        }

        $md5data = $method->paytr_general_merchant_id . $mb_data['transaction_id'] .
            strtoupper (md5 (trim($method->secret_word))) . $mb_data['mb_amount'] . $mb_data['mb_currency'] .
            $mb_data['status'];

        $calcmd5 = md5 ($md5data);
        if (strcmp (strtoupper ($calcmd5), $mb_data['md5sig'])) {
            return "MD5 checksum doesn't match - calculated: $calcmd5, expected: " . $mb_data['md5sig'];
        }

        return FALSE;
    }

    function _getPaymentResponseHtml ($paymentTable, $payment_name) {
        vmLanguage::loadJLang('com_virtuemart');
        $html = '<table>' . "\n";
        $html .= $this->getHtmlRow ('COM_VIRTUEMART_PAYMENT_NAME', $payment_name);
        if (!empty($paymentTable)) {
            $html .= $this->getHtmlRow (vmText::_('VMPAYMENT_PAYTR_ORDER_NUMBER'), $paymentTable->order_number);
        }
        $html .= '</table>' . "\n";
        return $html;
    }

    function _getInternalData ($virtuemart_order_id, $order_number = '') {
        $db = JFactory::getDBO ();
        $q = 'SELECT * FROM `' . $this->_tablename . '` WHERE ';
        if ($order_number) {
            $q .= " `order_number` = '" . $order_number . "'";
        } else {
            $q .= ' `virtuemart_order_id` = ' . $virtuemart_order_id;
        }
        $db->setQuery ($q);
        if (!($paymentTable = $db->loadObject ())) {
            return '';
        }
        return $paymentTable;
    }

    function _storeInternalData ($method, $request, $virtuemart_order_id) {
        $post_msg   = '';
        $database   = JFactory::getDBO();
        $query      = 'SHOW COLUMNS FROM `' . $this->_tablename . '` ';
        $database->setQuery($query);
        $columns    = $database->loadColumn(0);
        foreach ($request as $key => $value) {
            $post_msg .= $key . "=" . $value . "<br />";
            $table_key = 'paytr_' . $key;
            if (in_array ($table_key, $columns)) {
                $response_fields[$table_key] = $value;
            }
        }
        $response_fields['payment_name']                = 'PayTR';
        $response_fields['payment_method']              = 'paytr_'.strtolower($request['payment_type']);
        $response_fields['order_number']                = $this->normalizeOrderID($request['merchant_oid']);
        $response_fields['virtuemart_order_id']         = $virtuemart_order_id;
        $response_fields['virtuemart_paymentmethod_id'] = $method->virtuemart_paymentmethod_id;
        $this->storePSPluginInternalData ($response_fields, 'virtuemart_order_id', TRUE);
    }

    function getTableSQLFields () {
        return [
            'id'                            => 'int(11) UNSIGNED NOT NULL AUTO_INCREMENT',
            'virtuemart_order_id'           => 'int(1) UNSIGNED',
            'order_number'                  => ' char(64)',
            'virtuemart_paymentmethod_id'   => 'mediumint(1) UNSIGNED',
            'payment_name'                  => 'varchar(5000)',
            'payment_method'                => 'varchar(5000)',
            'payment_order_total'           => 'decimal(15,5) NOT NULL DEFAULT \'0.00000\'',
            'payment_currency'              => 'char(3) ',
            'cost_per_transaction'          => 'decimal(10,2)',
            'cost_percent_total'            => 'decimal(10,2)',
            'tax_id'                        => 'smallint(1)',
            'user_session'                  => 'varchar(255)',
            'paytr_hash'                    => 'varchar(50)',
            'paytr_merchant_oid'            => 'varchar(50)',
            'paytr_status'                  => 'varchar(50)',
            'paytr_total_amount'            => 'varchar(15)',
            'paytr_payment_type'            => 'varchar(15)',
            'paytr_payment_amount'          => 'varchar(15)',
            'paytr_currency'                => 'varchar(15)',
            'paytr_installment_count'       => 'int(11)',
            'paytr_merchant_id'             => 'int(11)',
            'paytr_test_mode'               => 'smallint(1)'
        ];
    }

    function plgVmConfirmedOrder ($cart, $order) {
        $method = $this->getVmPluginMethod ($order['details']['BT']->virtuemart_paymentmethod_id);
        if (!($method)){
            return NULL;
        }
        if (!$this->selectedThisElement ($method->payment_element)) {
            return FALSE;
        }
        $paytrHelper        = new PaytrHelper($method, $order);
        $paytrRequestHelper = new PaytrRequestHelper($paytrHelper->makePostVariables());
        $session = JFactory::getSession();
        $return_context = $session->getId ();
        $this->logInfo ('plgVmConfirmedOrder order number: ' . $order['details']['BT']->order_number, 'message');
        if (!class_exists ('VirtueMartModelOrders')) {
            require(VMPATH_ADMIN .'/models/orders.php');
        }
        if (!class_exists ('VirtueMartModelCurrency')) {
            require(VMPATH_ADMIN .'/models/currency.php');
        }
        if (!class_exists ('TableVendors')) {
            require(VMPATH_ADMIN .'/tables/vendors.php');
        }
        $this->getPaymentCurrency ($method);
        $totalInPaymentCurrency = vmPSPlugin::getAmountInCurrency($order['details']['BT']->order_total,$method->payment_currency);
        if ($totalInPaymentCurrency['value'] <= 0) {
            vmInfo (vmText::_ ('VMPAYMENT_PAYTR_PAYMENT_AMOUNT_INCORRECT'));
            return FALSE;
        }
        $dbValues['user_session'] = $return_context;
        $dbValues['order_number'] = $order['details']['BT']->order_number;
        $dbValues['payment_name'] = $this->renderPluginName ($method, $order);
        $dbValues['virtuemart_paymentmethod_id'] = $cart->virtuemart_paymentmethod_id;
        $dbValues['cost_per_transaction'] = $method->cost_per_transaction;
        $dbValues['cost_percent_total'] = $method->cost_percent_total;
        $dbValues['payment_currency'] = $method->payment_currency;
        $dbValues['payment_order_total'] = $totalInPaymentCurrency['value'];
        $dbValues['tax_id'] = $method->tax_id;
        $this->storePSPluginInternalData ($dbValues);
        $html = $this->renderByLayout('paytr_iframe', array(
            'token' => $paytrRequestHelper->getPaytrToken()['token']
        ));
        $cart->_confirmDone = FALSE;
        $cart->_dataValidated = FALSE;
        $cart->setCartIntoSession ();
        vRequest::setVar ('html', $html);
    }

    function plgVmgetPaymentCurrency ($virtuemart_paymentmethod_id, &$paymentCurrencyId) {
        if (!($method = $this->getVmPluginMethod ($virtuemart_paymentmethod_id))) {
            return NULL;
        }
        if (!$this->selectedThisElement ($method->payment_element)) {
            return FALSE;
        }
        $this->getPaymentCurrency ($method);
        $paymentCurrencyId = $method->payment_currency;
    }

    protected function getPluginHtml($plugin, $selectedPlugin, $pluginSalesPrice)
    {
        $this->setPluginConfig($plugin);
        $session = JFactory::getSession();
        $paytrSession = $session->get('paytr_pm', 0, 'vm');
        $paytrData['paytr_paymentmethod'] = false;

        if ($paytrSession)
        {
            $paytrData = json_decode($paytrSession, true);
        }
        $pluginMethodId = $this->_idName;
        $pluginName = $this->_psType . '_name';
        if (!class_exists('CurrencyDisplay'))
        {
            require VMPATH_ADMIN .'/helpers/currencydisplay.php';
        }
        $currency = CurrencyDisplay::getInstance();
        $costDisplay = "";

        if ($pluginSalesPrice)
        {
            $costDisplay = $currency->priceDisplay($pluginSalesPrice);
            $costDisplay = '<span class="' . $this->_type . '_cost"> (' .
                vmText::_('COM_VIRTUEMART_PLUGIN_COST_DISPLAY') . $costDisplay . ")</span>";
        }

        $html = '<div style="display:block">
        <input
            type="radio"
            name="'.$pluginMethodId.'"
            id="'.$this->_psType.'_id_'.$plugin->$pluginMethodId.'"
            value="'.$plugin->$pluginMethodId.'"
            '.(VmConfig::get('oncheckout_ajax', false) ? 'data-dynamic-update="1"' : '').' 
            checked
        >
            <label for="'.$this->_psType.'_id_'.$plugin->$pluginMethodId.'">
                <span class="'.$this->_type.'">'.$plugin->$pluginName.$costDisplay.'</span>
            </label>
        </div>';

        $html .= '<script type="text/javascript">
			var paymentSelection = document.querySelectorAll(".paytr-payment-selection");
			[].forEach.call(paymentSelection, function(elm){
				elm.addEventListener("click", function(){
					this.querySelector("input").click();
				});
			});
		</script>';
        return $html;
    }

    protected function checkConditions($cart, $activeMethod, $cart_prices) {
        return parent::checkConditions($cart, $activeMethod, $cart_prices);
    }

    public function setPaymentMethod($paymentMethod)
    {
        $this->paymentMethod = $paymentMethod;
    }

    public function setPluginConfig($pluginConfig)
    {
        $this->pluginConfig = $pluginConfig;
    }

    function plgVmOnPaymentResponseReceived (&$html) {
        if (!class_exists ('VirtueMartCart')) {
            require(VMPATH_SITE . DS . 'helpers' . DS . 'cart.php');
        }
        if (!class_exists ('shopFunctionsF')) {
            require(VMPATH_SITE . DS . 'helpers' . DS . 'shopfunctionsf.php');
        }
        if (!class_exists ('VirtueMartModelOrders')) {
            require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
        }
        vmLanguage::loadJLang('com_virtuemart_orders', TRUE);
        $mb_data = vRequest::getPost();
        $virtuemart_paymentmethod_id = vRequest::getInt ('pm', 0);
        $order_number = vRequest::getString ('on', 0);
        if (!($method = $this->getVmPluginMethod ($virtuemart_paymentmethod_id))) {
            return NULL;
        }
        if (!$this->selectedThisElement ($method->payment_element)) {
            return NULL;
        }
        if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber ($order_number))) {
            return NULL;
        }
        if (!($paymentTable = $this->getDataByOrderId ($virtuemart_order_id))) {
            return '';
        }
        vmLanguage::loadJLang('com_virtuemart_orders', TRUE);
        $orderModel = VmModel::getModel('orders');
        $order = $orderModel->getOrder($virtuemart_order_id);
        $payment_name = $this->renderPluginName ($method);
        $html = $this->_getPaymentResponseHtml ($paymentTable, $payment_name);
        $link=	JRoute::_("index.php?option=com_virtuemart&view=orders&layout=details&order_number=".$order['details']['BT']->order_number."&order_pass=".$order['details']['BT']->order_pass, false) ;
        $html .='<br />
		<a class="vm-button-correct" href="'.$link.'">'.vmText::_('COM_VIRTUEMART_ORDER_VIEW_ORDER').'</a>';
        $cart = VirtueMartCart::getCart();
        $amountInCurrency = vmPSPlugin::getAmountInCurrency($order['details']['BT']->order_total, $order['details']['BT']->payment_currency_id);
        CurrencyDisplay::getInstance($cart->pricesCurrency);
        $cart = VirtueMartCart::getCart();
        $cart->emptyCart();
        $html = $this->renderByLayout('paytr_success', array(
            'pageTitle'     => vmText::sprintf('VMPAYMENT_PAYTR_RESPONSE_PAGE_TITLE', $order['details']['BT']->order_number, $amountInCurrency['display']),
            'order_number'  => $order['details']['BT']->order_number,
            'order_pass'    => $order['details']['BT']->order_pass,
        ));
        return true;
    }

    function plgVmOnUserPaymentCancel () {
        if (!class_exists ('VirtueMartModelOrders')) {
            require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
        }
        $order_number = vRequest::getString ('on', '');
        $virtuemart_paymentmethod_id = vRequest::getInt ('pm', '');
        if (empty($order_number) or
            empty($virtuemart_paymentmethod_id) or
            !$this->selectedThisByMethodId ($virtuemart_paymentmethod_id)
        ) {
            return NULL;
        }

        if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber ($order_number))) {
            return NULL;
        }

        if (!($paymentTable = $this->getDataByOrderId ($virtuemart_order_id))) {
            return NULL;
        }
        VmInfo (vmText::_ ('VMPAYMENT_PAYTR_PAYMENT_CANCELLED'));
        $session = JFactory::getSession ();
        $return_context = $session->getId ();
        if (strcmp ($paymentTable->user_session, $return_context) === 0) {
            $this->handlePaymentUserCancel ($virtuemart_order_id);
        }
        return TRUE;
    }


    function normalizeOrderID($oid){
        $oid = explode('VM', $oid);
        $oid = explode('SP', $oid[0]);
        return $oid[1];
    }

    function paytrNotificationFilter($request){
        return [
            'hash'                  =>  $request['hash'],
            'merchant_oid'          =>  $request['merchant_oid'],
            'status'                =>  $request['status'],
            'total_amount'          =>  $request['total_amount'],
            'payment_type'          =>  $request['payment_type'],
            'payment_amount'        =>  $request['payment_amount'],
            'currency'              =>  $request['currency'],
            'installment_count'     =>  $request['installment_count'],
            'merchant_id'           =>  $request['merchant_id'],
            'test_mode'             =>  $request['test_mode']
        ];
    }

    function plgVmOnPaymentNotification () {
        if (!class_exists ('VirtueMartModelOrders')) {
            require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
        }
        $request = $this->paytrNotificationFilter(vRequest::getPost());
        if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($this->normalizeOrderID($request['merchant_oid'])))){
            return null;
        }
        if (!($payment = $this->getDataByOrderId($virtuemart_order_id))){
            return null;
        }
        $method = $this->getVmPluginMethod($payment->virtuemart_paymentmethod_id);
        if (!$this->selectedThisElement($method->payment_element)){
            return null;
        }
        if (!$payment){
            $this->logInfo ('getDataByOrderId payment not found', 'ERROR');
            return null;
        }
        $paytrHelper = new PaytrHelper($method);
        $hash = base64_encode( hash_hmac('sha256', $request['merchant_oid'].$paytrHelper->getMerchantSalt().$request['status'].$request['total_amount'], $paytrHelper->getMerchantKey(), true) );
        if($request['hash'] == $hash){
            $this->_storeInternalData($method, $request, $virtuemart_order_id);
            $modelOrder = VmModel::getModel ('orders');
            $vmorder = $modelOrder->getOrder ($virtuemart_order_id);
            $order = array();
            if ($request['status'] != 'success'){
                $order['customer_notified'] = 0;
                $order['order_status'] = $method->status_canceled;
                $order['comments'] = 'Payment Failed: ' . $this->normalizeOrderID($request['merchant_oid']);
                $modelOrder->updateStatusForOneOrder ($virtuemart_order_id, $order, TRUE);
                $this->logInfo('Payment Failed: ' . 'Payment Failed: ' . $this->normalizeOrderID($request['merchant_oid']), 'ERROR');
            }else{
                $this->logInfo('Payment Successful');
            }
            $order['customer_notified'] = 0;
            if ($request['status'] == 'success') {
                $order['paid'] = $request['total_amount'] / 100;
                $order['order_status'] = $method->status_success;
                $order['comments'] = vmText::sprintf('VMPAYMENT_PAYTR_PAYMENT_STATUS_CONFIRMED', $request['merchant_oid'].' ('.$this->normalizeOrderID($request['merchant_oid']).')');
            }else{
                $order['order_status'] = $method->status_canceled;
            }
            $order['virtuemart_order_id'] = $virtuemart_order_id;
            $this->logInfo ('PayTR Notification:' . $order['order_status']);
            $modelOrder->updateStatusForOneOrder($virtuemart_order_id, $order, true);
            $this->emptyCart ($payment->user_session, $this->normalizeOrderID($request['merchant_oid']));
        }else{
            $this->logInfo('PayTR Payment Failed: ' . print_r($request, true), 'ERROR');
        }
        echo 'OK';
        exit();
    }

    public function plgVmOnShowOrderBEPayment ($virtuemart_order_id, $payment_method_id) {
        if (!$this->selectedThisByMethodId ($payment_method_id)) {
            return NULL;
        }
        if (!($paymentTable = $this->_getInternalData ($virtuemart_order_id))) {
            return '';
        }
        $q = 'SELECT `currency_code_3` FROM `#__virtuemart_currencies` WHERE `virtuemart_currency_id`="' .
            $paymentTable->payment_currency . '" ';
        $db = JFactory::getDBO();
        $db->setQuery ($q);
        $currency_code_3 = $db->loadResult ();
        $html = '<table class="adminlist table">' . "\n";
        $html .= $this->getHtmlHeaderBE ();
        $html .= $this->getHtmlRowBE ('PAYMENT_NAME', $paymentTable->payment_name);
        $code = "mb_";
        foreach ($paymentTable as $key => $value) {
            if (substr ($key, 0, strlen ($code)) == $code) {
                if ($key == 'mb_pay_to_email' || $key == 'mb_transaction_id' || $key == 'PAYMENT_NAME') {
                    $html .= $this->getHtmlRowBE (vmText::_ (strtoupper($key)), $value);
                }
            }
        }
        $html .= '</table>' . "\n";
        return $html;
    }

    function plgVmOnStoreInstallPaymentPluginTable ($jplugin_id) {
        return $this->onStoreInstallPluginTable ($jplugin_id);
    }

    public function plgVmOnSelectCheckPayment (VirtueMartCart $cart,  &$msg)
    {
        $session = JFactory::getSession();
        $paytrSession = new stdClass;
        $paytrPaymentMethod = vRequest::getVar('paytr_paymentmethod', '');
        $paytrSession->virtuemart_payment_method_id = $cart->virtuemart_paymentmethod_id;
        $paytrSession->paytr_paymentmethod = $paytrPaymentMethod;
        $session->set('paytr_pm', json_encode($paytrSession), 'vm');
        return $this->OnSelectCheck($cart);
    }

    public function plgVmDisplayListFEPayment (VirtueMartCart $cart, $selected, &$htmlIn) {
        if ($this->getPluginMethods($cart->vendorId) === 0) {
            if (empty($this->_name)) {
                $app = JFactory::getApplication();
                $app->enqueueMessage(vmText::_('COM_VIRTUEMART_CART_NO_' . strtoupper($this->_psType)));
            }
            return false;
        }
        $method_name = $this->_psType . '_name';
        $idN = 'virtuemart_'.$this->_psType.'method_id';
        foreach ($this->methods as $this->_currentMethod) {
            if ($this->checkConditions($cart, $this->_currentMethod, $cart->cartPrices)) {
                $html = '';
                $cartPrices=$cart->cartPrices;
                if (isset($this->_currentMethod->cost_method)) {
                    $cost_method=$this->_currentMethod->cost_method;
                } else {
                    $cost_method=true;
                }
                $methodSalesPrice = $this->setCartPrices($cart, $cartPrices, $this->_currentMethod, $cost_method);
                $this->_currentMethod->payment_currency = $this->getPaymentCurrency($this->_currentMethod);
                $this->_currentMethod->$method_name = $this->renderPluginName($this->_currentMethod);
                $html .= $this->getPluginHtml($this->_currentMethod, $selected, $methodSalesPrice);
                $htmlIn[$this->_psType][$this->_currentMethod->$idN] =$html;
            }
        }
        return true;
    }

    public function plgVmonSelectedCalculatePricePayment (VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
        return $this->onSelectedCalculatePrice ($cart, $cart_prices, $cart_prices_name);
    }

    function plgVmOnCheckAutomaticSelectedPayment (VirtueMartCart $cart, array $cart_prices, &$paymentCounter) {
        return parent::onCheckAutomaticSelected ($cart, $cart_prices, $paymentCounter);
    }

    public function plgVmOnShowOrderFEPayment ($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name) {
        $this->onShowOrderFE ($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
    }

    function plgVmonShowOrderPrintPayment ($order_number, $method_id) {
        return $this->onShowOrderPrint ($order_number, $method_id);
    }

    public function plgVmOnUpdateOrderPayment(&$order, $oldOrderStatus) {
        if (!($method = $this->getVmPluginMethod($order->virtuemart_paymentmethod_id)))
        {
            return null;
        }
        if (!$this->selectedThisElement($method->payment_element))
        {
            return null;
        }
        if (!($payments = $this->getDatasByOrderId($order->virtuemart_order_id)))
        {
            return null;
        }
        $payment = $payments[0];
        $this->setPluginConfig($method);
        $this->setPaymentMethod($payment->payment_method);

        if($oldOrderStatus == 'C' && $order->order_status == 'R')
        {
            $paytrHelper = new PaytrHelper($method);
            $reference_no = utf8_strtoupper(md5(time()));
            $postVariables = [
                'merchant_id'   => $paytrHelper->getMerchantID(),
                'merchant_oid'  => $payment->paytr_merchant_oid,
                'return_amount' => $payment->paytr_total_amount / 100,
                'reference_no'  => $reference_no
            ];
            $paytr_token = base64_encode(hash_hmac('sha256',$postVariables['merchant_id'].$postVariables['merchant_oid'].$postVariables['return_amount'].$paytrHelper->getMerchantSalt(),$paytrHelper->getMerchantKey(),true));
            $paytrRequest = new PaytrRequestHelper(array_merge($postVariables, ['paytr_token' => $paytr_token]));
            $result = $paytrRequest->doRefund();
            if ($result['status'] != 'success') {
                vmError (vmText::_ ('VMPAYMENT_PAYTR_REFUND_FAILED'));
                return FALSE;
            }else{
                return TRUE;
            }
        }
        else
        {
            return NULL;
        }
    }

    function plgVmDeclarePluginParamsPaymentVM3( &$data) {

        return $this->declarePluginParams('payment', $data);
    }

    function plgVmSetOnTablePluginParamsPayment ($name, $id, &$table) {
        return $this->setOnTablePluginParams ($name, $id, $table);
    }

}
